package com.nokia.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.nokia.dao.CityDao;
import com.nokia.dao.UserDao;
import com.nokia.dao.UserRoleCityDao;
import com.nokia.dao.UserRoleDao;
import com.nokia.pojo.City;
import com.nokia.pojo.Role;
import com.nokia.pojo.User;
import com.nokia.pojo.UserRole;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class UserService extends ServiceImpl<UserDao, User> {

    private final UserDao userDao;
    private final UserRoleDao userRoleDao;
    private final UserRoleCityDao userRoleCityDao;
    private final CityDao cityDao;

    @Autowired
    public UserService(UserDao userDao, UserRoleDao userRoleDao, CityDao cityDao, UserRoleCityDao userRoleCityDao) {
        this.userDao = userDao;
        this.userRoleDao = userRoleDao;
        this.cityDao = cityDao;
        this.userRoleCityDao = userRoleCityDao;
    }

    /**
     * 添加用户，不考虑权限
     * 
     * @param user
     * @return
     */
    public boolean addUser(User user) {
        // 检查是否已存在同名用户
        if (hasLoginName(user.getLoginName())) {
            log.debug("", user.getLoginName());
            return false;
        }
        userDao.insert(user);
        return true;
    }

    /**
     * 指定的登陆名是否已存在
     * 
     * @param loginName
     * @return
     */
    public boolean hasLoginName(String loginName) {
        return null != userDao.hasLoginName(loginName);
    }

    /**
     * 用户是否存在某个权限
     * 
     * @param user
     * @param role
     * @param city
     * @return
     */
    public boolean hasRoleCity(User user, Role role, City city) {
        return null != userRoleCityDao.hasRoleCity(user.getUserId(), role.getRoleId(), city.getCityId());
    }

    /**
     * 给用户添加权限
     * 
     * @param user
     * @param role
     * @return
     */
    public String addRole(User user, Role role, City city) {
        if (!hasRole(user, role)) {
            // 检查用户，确认当前无此权限
            UserRole userRoleEntity = new UserRole();
            userRoleEntity.setUserId(user.getUserId());
            userRoleEntity.setRoleId(role.getRoleId());
            userRoleDao.insert(userRoleEntity);
        }
        return "ok";
    }

    /**
     * 判断用户是否有有某个角色
     * 
     * @param user
     * @param role
     * @return boolean
     */
    public boolean hasRole(User user, Role role) {
        return null != userRoleDao.hasRole(user.getUserId(), role.getRoleId());
    }

    /**
     * 添加用户
     * 
     * @param user
     * @return
     */
    @Transactional
    public String add(User user) {
        User query = new User();
        query.setLoginName(user.getLoginName());
        User one = baseMapper.selectOne(Wrappers.query(query));
        if (one != null) {
            return "loginName: " + user.getLoginName() + " 已存在";
        } else {
            user.setCityId(user.getCity().getCityId());
            baseMapper.insert(user);
            // Set<Role> roles = user.getRoles();
            // UserRoleCity userRoleCity;
            // for (Role role : roles) {
            // for (Integer cityId : role.getCities()) {
            // userRoleCity = new UserRoleCity();
            // userRoleCity.setRoleId(role.getRoleId());
            // userRoleCity.setUserId(user.getUserId());
            // userRoleCity.setCityId(cityId);
            // userRoleCityDao.insert(userRoleCity);
            // }
            // }
        }
        return "ok";
    }

    @Transactional
    public String delete(Integer userId) {
        // UserEntity userEntity = baseMapper.selectById(userId);
        /* cityDao.deleteById(userEntity.getCityId()); */
        QueryWrapper<UserRole> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRole> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        for (UserRole userRoleEntity : userRoleEntities) {
            Integer id = userRoleEntity.getId();
            userRoleDao.deleteById(id);
        }
        baseMapper.deleteById(userId);
        return "ok";
    }

    /**
     * 更新用户
     * 
     * @param user
     * @return
     */
    @Transactional
    public boolean update(User user) {
        // 更新user表
        baseMapper.updateById(user);
        // 更新user_role 分为2步
        Integer userId = user.getUserId();
        QueryWrapper<UserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        // 1. 删除原user_role
        userRoleDao.delete(queryWrapper);
        // 2. 添加新的user_role
        for (Role role : user.getRoles()) {
            UserRole userRoleEntity = new UserRole();
            userRoleEntity.setUserId(user.getUserId());
            userRoleEntity.setRoleId(role.getRoleId());
            userRoleDao.insert(userRoleEntity);
        }
        return true;
    }

    /**
     * 通过用户的登录名获取用户完整信息
     * 
     * @param loginName 用户登录名
     * @return UserEntity
     */
    public User getByLoginName(String loginName) {
        return userDao.getByLoginName(loginName);
    }

    public Map<String, Object> select(Integer userId) {
        Map<String, Object> hashMap = new HashMap<>();
        if (userId == null) {
            hashMap.put("userId", "为空");
            return hashMap;
        }
        User userEntity = baseMapper.selectById(userId);
        QueryWrapper<User> userEntityQueryWrapper = new QueryWrapper<>();
        userEntityQueryWrapper.eq("user_id", userId);
        User one = baseMapper.selectOne(userEntityQueryWrapper);
        if (one == null) {
            hashMap.put("userId", "不正确");
            return hashMap;
        }
        Integer cityId = userEntity.getCityId();
        City cityEntity = cityDao.selectById(cityId);
        // 他有两个角色 对 我知道 用for遍历来查 不需要for遍历查
        QueryWrapper<UserRole> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("user_id", userId);
        List<UserRole> userRoleEntities = userRoleDao.selectList(objectQueryWrapper);
        hashMap.put("userEntity", userEntity);
        hashMap.put("cityEntity", cityEntity);
        hashMap.put("userRoleEntities", userRoleEntities);
        return hashMap;
    }

}
